// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_close.c $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: source file

#include "main.h"



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_api_close(int fd)
{
	os_file_t*	fp		= NULL;
	os_mbox_t*	mbox	= NULL;
	os_msg_t*	msg		= NULL;
	int			ret;
	WORD		size;
	BOOLEAN		sts;
	int			unlock	= 0;

	for (;;)	// a convenience loop
	{
		if (os_api_global.initialized == 0)
		{
			// The library has not been initialized.
			ret	= -EPROTO;
			break;
		}

		// Acquire file access.
		ret	= os_file_lock(fd, &fp);

		if (ret)
		{
			// There was a problem!
			break;
		}

		unlock	= 1;

		// Begin the CLOSE operation.

		// Acquire a mailbox for the receive message.
		ret	= os_mbox_alloc(&mbox);

		if (ret)
			break;

		// Allocate a message structure.
		ret	= os_msg_alloc(&msg);

		if (ret)
			break;

		// Initialize and send the message.
		memset(msg, 0, sizeof(os_msg_t));
		msg->status		= OS_MSG_STS_OK;
		msg->msg_type	= OS_MSG_TYPE_CLOSE;
		msg->size		= OS_MSG_SIZE_CLOSE;
		msg->mbox		= mbox->mbox;
		sts				= SendRtData(fp->dev_mbox, msg, OS_MSG_SIZE_CLOSE);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			break;
		}

		// Get and process the response.
		size	= ReceiveRtData(mbox->mbox, msg, WAIT_FOREVER);

		if (size == 0)
		{
			// The operation failed.
			ret	= os_get_error_status();
			break;
		}

		if ((size != OS_MSG_SIZE_CLOSE) || (size != msg->size))
		{
			ret	= -EMSGSIZE;
			break;
		}

		switch (msg->status)
		{
			default:
			case OS_MSG_STS_MSG_TYPE:

				ret	= -EBADMSG;
				break;

			case OS_MSG_STS_OK:
			case OS_MSG_STS_MEM_MAP:
			case OS_MSG_STS_MSG_SIZE:

				ret	= msg->u.close.ret;
				break;

			case OS_MSG_STS_THREAD:

				ret	= -ENOEXEC;
				break;
		}

		unlock	= 0;
		break;
	}

	// Cleanup before leaving.
	os_msg_free(&msg);
	os_mbox_free(&mbox);

	if (unlock)
		os_file_unlock(&fp);
	else
		os_file_destroy(&fp);

	return(ret);
}


