// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/main.h $
// $Rev: 43267 $
// $Date: 2018-07-09 17:24:11 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Device Driver: header file

#ifndef __MAIN_H__
#define __MAIN_H__

#include "16ao4mf.h"

#define DEV_BAR_SHOW						0
#define DEV_METRICS_SHOW					0
#define DEV_PCI_ID_SHOW						0
#define DEV_SUPPORTS_IRQ
#define DEV_SUPPORTS_WAIT
#define DEV_SUPPORTS_WRITE
#define GSC_ALT_DEV_GET(a)					(a)->dev
#define GSC_ALT_STRUCT_GET(i,d)				gsc_alt_struct_get((i),(d))
#define GSC_ALT_STRUCT_T					chan_data_t
#define GSC_DEVS_PER_BOARD					4
typedef struct _dev_data_t					dev_data_t;
typedef struct _chan_data_t					chan_data_t;

#include "gsc_main.h"



// #defines *******************************************************************

#define DEV_MODEL							"16AO4MF"	// Upper case form of the below.
#define DEV_NAME							"16ao4mf"	// MUST AGREE WITH AISS8AO4_BASE_NAME

#define DEV_VERSION							"3.3"	// FOR DEVICE SPECIFIC CODE ONLY!
// 3.3	Changed "printk" calls to "printf" for cross platform compatibility.
// 3.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Updated initial file content for consistency.
//		Enabled metrics support.
//		Open now succeeds even if initialization fails.
// 3.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Minor correction to initialization code.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
// 3.0	Updated per changes to the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		LINTed the sources.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Migrated dev_irq_open()/close() to gsc_irq_open()/close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Fixed a bug in the shutdown code.
// 2.0	Driver overhauled and rewritten.

#define DEV_QTY_MAX							32

#define DEV_WAIT_GSC_ALL					AO4MF_WAIT_GSC_ALL
#define DEV_WAIT_ALT_ALL					AO4MF_WAIT_ALT_ALL

#define GSC_WRITE_PIO_WORK_32_BIT

// I/O services
#define DEV_PIO_WRITE_AVAILABLE				pio_write_available
#define DEV_PIO_WRITE_WORK					gsc_write_pio_work_32_bit
#define DEV_DMA_WRITE_AVAILABLE				dma_write_available
#define DEV_DMA_WRITE_WORK					dma_write_work
#define DEV_DMDMA_WRITE_AVAILABLE			dmdma_write_available
#define DEV_DMDMA_WRITE_WORK				dmdma_write_work



// data types *****************************************************************

struct _dev_io_t
{
	os_sem_t			sem;				// Only one Tx at a time.

	int					abort;
	size_t				bytes_per_sample;	// Sample size in bytes.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	s32					io_mode;			// PIO, DMA, DMDMA
	u32					io_reg_offset;		// Offset of board's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of board's I/O FIFO.
	os_mem_t			mem;				// I/O buffer.
	int					non_blocking;		// Is this non-blocking I/O?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
};

struct _chan_data_t
{
	os_sem_t			sem;		// control process access
	struct _dev_data_t*	dev;
	int					index;		// 0-3
	int					dev_index;	// 0-3, 4-7, ...
	int					users;		// Number of currently active open() requests.
	int					share;		// Were we opened in shared mode?

	dev_io_t			tx;

	gsc_wait_node_t*	wait_list;

	struct
	{
		s32				reg_diopcr;		// Digital I/O Port Control Register: Supported?

		s32				sync_clk_out;	// Are SYNC OUT and CLK OUT supported?

	} cache;

	struct
	{
		VADDR_T			gsc_accr_32;	// Adjustable Clock Control Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_bor_32;		// Buffer Operations Register
		VADDR_T			gsc_diopcr_32;	// Digital I/O Port Control Register
		VADDR_T			gsc_icscr_32;	// Internal Clock Source Control Register
		VADDR_T			gsc_odbr_32;	// Output Data Buffer Register
		VADDR_T			gsc_srcr_32;	// Sample Rate Control Register
	} vaddr;

};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic board type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the board being accessed.
	int					in_use;			// Only one user at a time.

	gsc_bar_t			bar;			// device register mappings
	chan_data_t			channel[GSC_DEVS_PER_BOARD];
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.

	struct
	{
		s32				auto_cal_ms;	// Max Auto Cal period in ms.

		s32				channel_qty;	// 4

		s32				fref_adj;		// The adjustable reference frequency.
		s32				fref_master;	// The master reference frequency.
		s32				fsamp_max;		// Maximum sample rate.
		s32				fsamp_min;		// Minimum sample rate.

		s32				init_ms;		// Max Init period in ms.

		s32				nclk_max;		// Maximum Nclk value.
		s32				nclk_min;		// Minimum Nclk value.
		s32				nrate_max;		// Maximum Nrate value.
		s32				nrate_min;		// Minimum Nrate value.

	} cache;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register
	} vaddr;

};



// prototypes *****************************************************************

void	dev_io_close(GSC_ALT_STRUCT_T* alt);
int		dev_io_create(dev_data_t* dev, int channel);
void	dev_io_destroy(dev_data_t* dev, int channel);
int		dev_io_open(GSC_ALT_STRUCT_T* alt);
int		dev_irq_create(dev_data_t* dev);
void	dev_irq_destroy(dev_data_t* dev);
long	dma_write_available(GSC_ALT_STRUCT_T* alt, size_t count);
long	dma_write_work(GSC_ALT_STRUCT_T* alt, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
long	dmdma_write_available(GSC_ALT_STRUCT_T* alt, size_t count);
long	dmdma_write_work(GSC_ALT_STRUCT_T* alt, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

int		initialize_ioctl(chan_data_t* chan, s32* arg);
void	irq_reg_mod(chan_data_t* chan, VADDR_T va, u32 value, u32 mask);

long	pio_write_available(GSC_ALT_STRUCT_T* alt, size_t count);



#endif
