// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_exit.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
// Send each device mailbox an exit message.
static void _dev_threads_exit(void)
{
	int			i;
	int			j;
	RTHANDLE	mbox;
	os_msg_t	msg;
	char		name[64];
	int			qty		= SIZEOF_ARRAY(gsc_global.dev_list) * GSC_DEVS_PER_BOARD;
	RTHANDLE	root;
	WORD		type	= 0;

	root	= os_get_root_process(NULL);

	if (root == BAD_RTHANDLE)
	{
		db_printf("%s: Failed to access root process.\n", DEV_NAME);
	}
	else
	{
		for (i = 0; i < qty; i++)
		{
			sprintf(name, "%s.%d", DEV_NAME, i);
			mbox	= LookupRtHandle(root, name, NO_WAIT);

			if (mbox == BAD_RTHANDLE)
				continue;

			memset(&msg, 0, sizeof(msg));
			msg.status		= OS_MSG_STS_OK;
			msg.msg_type	= OS_MSG_TYPE_EXIT;
			msg.size		= OS_MSG_SIZE_EXIT;
			SendRtData(mbox, &msg, msg.size);

			// Wait til the mailbox has been deleted.

			for (j = 0; j < 1000; j++)
			{
				os_time_sleep_ms(10);
				mbox	= LookupRtHandle(root, name, NO_WAIT);

				if (mbox == BAD_RTHANDLE)
					break;
			}
		}
	}
}



//*****************************************************************************
// Make call to close each device.
static void _close_each_device(void)
{
	int	i;
	int	ret;

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.dev_list); i++)
	{
		if (gsc_global.dev_list[i] == NULL)
			continue;

#if (GSC_DEVS_PER_BOARD > 1)

		{
			int	j;

			for (j = 0; j < GSC_DEVS_PER_BOARD; j++)
			{
				ret	= gsc_close(&gsc_global.dev_list[i]->channel[j]);

				if (ret)
					db_printf("%s: Failed generic closure on #%d.%d.\n", DEV_NAME, i, j);
			}
		}

#else

		ret	= gsc_close(gsc_global.dev_list[i]);

		if (ret)
			db_printf("%s: Failed generic closure on #%d.\n", DEV_NAME, i);

#endif
	}
}



//*****************************************************************************
// Send each service thread an exit message.
static void _service_threads_exit(void)
{
	int			i;
	int			j;
	RTHANDLE	mbox;
	os_msg_t	msg;

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.service_threads.data); i++)
	{
		if (gsc_global.service_threads.data[i].initialized == 0)
			continue;

		// Wait for the thread to finish working, if possible.

		for (j = 0; j < 1000; j++)
		{
			if (gsc_global.service_threads.data[i].initialized)
				os_time_sleep_ms(10);
			else
				break;
		}

		// Send the Exit message.

		memset(&msg, 0, sizeof(msg));
		msg.status		= OS_MSG_STS_OK;
		msg.msg_type	= OS_MSG_TYPE_EXIT;
		msg.size		= OS_MSG_SIZE_EXIT;
		mbox			= gsc_global.service_threads.data[i].mbox;
		SendRtData(mbox, &msg, msg.size);

		// Allow time for the thread to finish working, if possible.

		for (j = 0; j < 1000; j++)
		{
			os_time_sleep_ms(10);

			if (gsc_global.service_threads.data[i].mbox == BAD_RTHANDLE)
				break;
		}
	}
}



//*****************************************************************************
// Close down everything and finish exit cleanup.
void os_exit_driver(void)
{
	dev_data_t*	dev;
	int			i;

	for (;;)	// A convenience loop.
	{
		if ((gsc_global.driver_loaded == 0) && (gsc_global.dev_qty == 0))
		{
			// There is nothing for us to do.
			break;
		}

		os_sem_lock(&gsc_global.sem);

		_dev_threads_exit();
		_close_each_device();
		_service_threads_exit();

		for (i = 0; i < SIZEOF_ARRAY(gsc_global.dev_list); i++)
		{
			dev	= gsc_global.dev_list[i];

			if (dev)
			{
				gsc_init_dev_destroy(dev);
				gsc_init_dev_data_t_free(&dev);
			}
		}

		os_ioctl_reset();
		gsc_global.dev_qty	= 0;

		if (gsc_global.driver_loaded)
		{
			gsc_global.driver_loaded	= 0;
			printf(	"%s: driver version %s successfully removed.\n",
					DEV_NAME,
					GSC_DRIVER_VERSION);
		}

		os_sem_destroy(&gsc_global.service_threads.sem);
		break;
	}
}



//*****************************************************************************
// Send en EXIT message to the Driver Mailbox.
int os_exit_initiate(const char* prefix)
{
	int			i;
	RTHANDLE	mbox;
	os_msg_t	msg;
	char		name[64];
	int			qty;
	int			ret;
	RTHANDLE	root;
	BOOLEAN		sts;

	for (;;)	// A convenience loop.
	{
		ret	= os_get_root_process(&root);

		if (ret)
			break;

		mbox	= LookupRtHandle(root, DEV_NAME, NO_WAIT);

		if (mbox == BAD_RTHANDLE)
		{
			printf("%s", prefix);
			prefix	= "";
			ret		= os_get_error_status();
			db_printf("%s: Failed to access Driver Mailbox.\n", DEV_NAME);
			db_printf("%s: Driver may not be running.\n", DEV_NAME);
			break;
		}

		memset(&msg, 0, sizeof(msg));
		msg.status		= OS_MSG_STS_OK;
		msg.msg_type	= OS_MSG_TYPE_EXIT;
		msg.size		= OS_MSG_SIZE_EXIT;
		sts	= SendRtData(mbox, &msg, msg.size);

		if (sts != FALSE)
			break;

		ret	= 1;
		printf("%s", prefix);
		os_get_error_status();
		db_printf("%s: Failed to send EXIT message to Driver Mailbox.\n", DEV_NAME);
		db_printf("%s: Initiating exhaustive termination.\n", DEV_NAME);

		// This likely means the Driver Mailbox exists,
		// but that the driver was not terminate properly.
		// So, we'll uncatalog the Driver Mailbox.
		UncatalogRtHandle(root, DEV_NAME);

		// For completeness, we'll try to close every device and uncatalog its mailbox.
		qty	= SIZEOF_ARRAY(gsc_global.dev_list) * GSC_DEVS_PER_BOARD;

		for (i = 0; i < qty; i++)
		{
			sprintf(name, "%s.%d", DEV_NAME, i);
			mbox	= LookupRtHandle(root, name, NO_WAIT);

			if (mbox == BAD_RTHANDLE)
			{
				// The mailbox doesn't exist.
				continue;
			}

			// Send a CLOSE message to the mailbox.
			memset(&msg, 0, sizeof(os_msg_t));
			msg.status		= OS_MSG_STS_OK;
			msg.msg_type	= OS_MSG_TYPE_CLOSE;
			msg.size		= OS_MSG_SIZE_CLOSE;
			msg.mbox		= BAD_RTHANDLE;
			SendRtData(mbox, &msg, OS_MSG_SIZE_CLOSE);

			// Send an EXIT message to the mailbox.
			memset(&msg, 0, sizeof(msg));
			msg.status		= OS_MSG_STS_OK;
			msg.msg_type	= OS_MSG_TYPE_EXIT;
			msg.size		= OS_MSG_SIZE_EXIT;
			SendRtData(mbox, &msg, msg.size);

			// Uncatalog the mailbox.
			UncatalogRtHandle(root, name);
		}

		break;
	}

	return(ret);
}


