// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_init.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
int os_init_driver(void)
{
	dev_data_t*	dev		= NULL;
	RTHANDLE	handle;
	int			pri_max	= 4;	// value from INtime tech support
	PCIDEV		pcidev;
	int			ret		= 0;
	SYSINFO		si;
	int			size	= sizeof(dev_data_t);
	int			sts;
	BOOLEAN		tst;

	PciInitialize(0);
	os_sem_create(&gsc_global.sem);

	// Compute time information.
	CopyRtSystemInfo(&si);
	gsc_global.tick_rate	= 1000 / si.NucleusTickInterval * si.KernelTickRatio;
	gsc_global.tick_us		= 1000000L / gsc_global.tick_rate;

	// Adjust the maximum process priority. The value "4" is from INtime tech
	// support, and is specifically because we use interrupt support threads.
	handle	= GetRtThreadHandles(THIS_PROCESS);
	tst		= SetRtProcessMaxPriority(handle, pri_max);

	if (tst == FALSE)
	{
		// The max priority could not be adjusted.
		// We'll report it and ignore it.
		db_printf("%s: Failed to adjust maximum process priority.\n", DEV_NAME);
		db_printf("%s: Maximum priority requested: %d\n", DEV_NAME,  pri_max);
	}

	// Locate the devices and add them to our list.
	memset(&pcidev, 0, sizeof(pcidev));
	pcidev.wVendorId	= 0x10B5;	// PLX
	pcidev.wDeviceId	= 0;		// Find all PLX devices.
	pcidev.wDeviceIndex	= 0;		// Find the first device.

	for (;;)
	{
		ret	= gsc_init_dev_data_t_alloc(&dev);

		if (ret < 0)
			break;	// The request failed.

		if (dev == NULL)
		{
			// This should never happen!
			ret	= -EFAULT;
			break;
		}

		memset(dev, 0, size);
		memcpy(&dev->pci.pcidev, &pcidev, sizeof(pcidev));
		sts	= PciFindDevice(&dev->pci.pcidev);

		if (sts == FALSE)
			break;			// A device was not found.

		ret	= gsc_init_dev_create(dev);

		if (ret > 0)
		{
			// The referenced device was added.
			printf("%s: device loaded:  %s\n", DEV_NAME, dev->model);
			dev	= NULL;
		}
		else if (ret == 0)
		{
			// The referenced device was not ours.
			gsc_init_dev_destroy(dev);
			gsc_init_dev_data_t_free(&dev);
		}
		else
		{
			// The referenced device was not added.
			// An error should have been reported.
			gsc_init_dev_destroy(dev);
			gsc_init_dev_data_t_free(&dev);
			ret	= 0;
			printf("%s: Failed to add device.\n", DEV_NAME);
		}

		pcidev.wDeviceIndex++;	// Find the next device.
	}

	if (dev)
	{
		gsc_init_dev_destroy(dev);
		gsc_init_dev_data_t_free(&dev);
		dev	= NULL;
	}

	// Perform initialization following device discovery.

	for (;;)	// A convenience loop.
	{
		if (gsc_global.dev_qty <= 0)
		{
			ret	= -ENODEV;
			printf(	"%s: driver load failure: version %s\n",
					DEV_NAME,
					GSC_DRIVER_VERSION);
			break;
		}

		ret	= gsc_ioctl_init();

		if (ret)
		{
			if (ret)
				db_printf("%s: Failed IOCTL initialization.\n", DEV_NAME);

			break;
		}

		ret	= os_mbox_devices_start();

		if (ret)
		{
			if (ret)
				db_printf("%s: Failed creation of device mailboxes.\n", DEV_NAME);

			break;
		}

		gsc_global.driver_loaded	= 1;
		printf(	"%s: driver loaded:  version %s\n",
				DEV_NAME,
				GSC_DRIVER_VERSION);
		ret	= 0;
		break;
	}

	return(ret);
}


