// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_reg.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
u8 os_reg_mem_mx_u8(dev_data_t* dev, VADDR_T va, u8 value, u8 mask)
{
	u8	reg	= 0xEF;	// lower portion of DEADBEEF
	u8	set;

	// There is a special case for reads of register id value 0. This maps to
	// an 8-bit, BAR0 register. At this level we see it as va == 0.

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
	{
		reg	= ((volatile u8*) va)[0];
		set	= (reg & ~mask) | (value & mask);
		((volatile u8*) va)[0]	= set;
	}

	if (dev)
		gsc_irq_access_unlock(dev);

	return(reg);
}



//*****************************************************************************
u16 os_reg_mem_mx_u16(dev_data_t* dev, VADDR_T va, u16 value, u16 mask)
{
	u16	reg	= 0xBEEF;	// lower portion of DEADBEEF
	u16	set;

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
	{
		reg	= ((volatile u16*) va)[0];
		set	= (reg & ~mask) | (value & mask);
		((volatile u16*) va)[0]	= set;
	}

	if (dev)
		gsc_irq_access_unlock(dev);

	return(reg);
}



//*****************************************************************************
u32 os_reg_mem_mx_u32(dev_data_t* dev, VADDR_T va, u32 value, u32 mask)
{
	u32	reg	= 0xDEAFBEEF;
	u32	set;

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
	{
		reg= ((volatile u32*) va)[0];
		set	= (reg & ~mask) | (value & mask);
		((volatile u32*) va)[0]	= set;
	}

	if (dev)
		gsc_irq_access_unlock(dev);

	return(reg);
}



//*****************************************************************************
u8 os_reg_mem_rx_u8(dev_data_t* dev, VADDR_T va)
{
	u8	value	= 0xEF;	// lower portion of DEADBEEF

	// There is a special case for reads of register id value 0. This maps to
	// an 8-bit, BAR0 register. At this level we see it as va == 0.

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		value	= ((volatile u8*) va)[0];

	if (dev)
		gsc_irq_access_unlock(dev);

	return(value);
}



//*****************************************************************************
u16 os_reg_mem_rx_u16(dev_data_t* dev, VADDR_T va)
{
	u16	value	= 0xBEEF;	// lower portion of DEADBEEF

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		value	= ((volatile u16*) va)[0];

	if (dev)
		gsc_irq_access_unlock(dev);

	return(value);
}



//*****************************************************************************
u32 os_reg_mem_rx_u32(dev_data_t* dev, VADDR_T va)
{
	u32	value	= 0xDEADBEEF;

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		value	= ((volatile u32*) va)[0];

	if (dev)
		gsc_irq_access_unlock(dev);

	return(value);
}



//*****************************************************************************
void os_reg_mem_tx_u8(dev_data_t* dev, VADDR_T va, u8 value)
{
	// There is a special case for reads of register id value 0. This maps to
	// an 8-bit, BAR0 register. At this level we see it as va == 0.

	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		((volatile u8*) va)[0]	= value;

	if (dev)
		gsc_irq_access_unlock(dev);
}



//*****************************************************************************
void os_reg_mem_tx_u16(dev_data_t* dev, VADDR_T va, u16 value)
{
	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		((volatile u16*) va)[0]	= value;

	if (dev)
		gsc_irq_access_unlock(dev);
}



//*****************************************************************************
void os_reg_mem_tx_u32(dev_data_t* dev, VADDR_T va, u32 value)
{
	if (dev)
		gsc_irq_access_lock(dev);

	if (va)
		((volatile u32*) va)[0]	= value;

	if (dev)
		gsc_irq_access_unlock(dev);
}



//*****************************************************************************
u8 os_reg_pci_mx_u8(dev_data_t* dev, int lock, u16 offset, u8 value, u8 mask)
{
	DWORD	reg	= 0xEF;	// lower portion of DEADBEEF

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		reg	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_BYTE);
		reg	= (reg & ~mask) | (value & mask);
		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_BYTE, reg);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return((u8) reg);
}



//*****************************************************************************
u16 os_reg_pci_mx_u16(dev_data_t* dev, int lock, u16 offset, u16 value, u16 mask)
{
	DWORD	reg	= 0xBEEF;	// lower portion of DEADBEEF

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		reg	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_WORD);
		reg	= (reg & ~mask) | (value & mask);
		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_WORD, reg);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return((u16) reg);
}



//*****************************************************************************
u32 os_reg_pci_mx_u32(dev_data_t* dev, int lock, u16 offset, u32 value, u32 mask)
{
	DWORD	reg	= 0xDEADBEEF;
	u32		set;

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		reg	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_DWORD);
		set	= (reg & ~mask) | (value & mask);
		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_DWORD, set);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return(reg);
}



//*****************************************************************************
u8 os_reg_pci_rx_u8(dev_data_t* dev, int lock, u16 offset)
{
	DWORD	dw;
	u8		reg	= 0xEF;	// lower portion of DEADBEEF

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		dw	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_BYTE);
		reg	= (u8) (dw & 0xFF);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return(reg);
}



//*****************************************************************************
u16 os_reg_pci_rx_u16(dev_data_t* dev, int lock, u16 offset)
{
	DWORD	dw;
	u16		reg	= 0xBEEF;	// lower portion of DEADBEEF

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		dw	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_WORD);
		reg	= (u16) (dw & 0xFFFF);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return(reg);
}



//*****************************************************************************
u32 os_reg_pci_rx_u32(dev_data_t* dev, int lock, u16 offset)
{
	u32	reg	= 0xDEADBEEF;

	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		reg	= PciGetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_DWORD);

		if (lock)
			gsc_irq_access_unlock(dev);
	}

	return(reg);
}



//*****************************************************************************
void os_reg_pci_tx_u8(dev_data_t* dev, int lock, u16 offset, u8 value)
{
	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_BYTE, value);

		if (lock)
			gsc_irq_access_unlock(dev);
	}
}



//*****************************************************************************
void os_reg_pci_tx_u16(dev_data_t* dev, int lock, u16 offset, u16 value)
{
	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_WORD, value);

		if (lock)
			gsc_irq_access_unlock(dev);
	}
}



//*****************************************************************************
void os_reg_pci_tx_u32(dev_data_t* dev, int lock, u16 offset, u32 value)
{
	if (dev)
	{
		if (lock)
			gsc_irq_access_lock(dev);

		PciSetConfigRegister(&dev->pci.pcidev, (BYTE) offset, T_DWORD, value);

		if (lock)
			gsc_irq_access_unlock(dev);
	}
}


