// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_spinlock.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// Linux: Device Driver: source file: This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).

#include "main.h"



/******************************************************************************
*
*	Function:	os_spinlock_create
*
*	Purpose:
*
*		Create a spin lock.
*
*	Arguments:
*
*		lock	The OS specific data type for the lock.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int os_spinlock_create(os_spinlock_t* lock)
{
	int	ret;

	ret	= os_sem_create(lock);

	if (ret)
		db_printf("%s: Failed to create spinlock.\n", DEV_NAME);

	return(ret);
}



/******************************************************************************
*
*	Function:	os_spinlock_destroy
*
*	Purpose:
*
*		Destroy a spin lock.
*
*	Arguments:
*
*		lock	The OS specific data type for the lock.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int os_spinlock_destroy(os_spinlock_t* lock)
{
	int	ret;

	ret	= os_sem_destroy(lock);

	if (ret)
		db_printf("%s: Failed to destroy spinlock.\n", DEV_NAME);

	return(ret);
}



/******************************************************************************
*
*	Function:	os_spinlock_lock
*
*	Purpose:
*
*		Acquire a spin lock.
*
*	Arguments:
*
*		lock	The OS specific data type for the lock.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int os_spinlock_lock(os_spinlock_t* lock)
{
	int	ret;

	ret	= os_sem_lock(lock);

	if (ret)
		db_printf("%s: Failed to lock spinlock.\n", DEV_NAME);

	return(ret);
}



/******************************************************************************
*
*	Function:	os_spinlock_unlock
*
*	Purpose:
*
*		Release a spin lock.
*
*	Arguments:
*
*		lock	The OS specific data type for the lock.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int os_spinlock_unlock(os_spinlock_t* lock)
{
	int	ret;

	ret	= os_sem_unlock(lock);

	if (ret)
		db_printf("%s: Failed to unlock spinlock.\n", DEV_NAME);

	return(ret);
}


