// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/write.c $
// $Rev: 43266 $
// $Date: 2018-07-09 17:21:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Device Driver: source file

#include "main.h"



//*****************************************************************************
static u32 _buffer_size(chan_data_t* chan)
{
	static const u32	list[16]	=
	{
		4,			8,			16,			32,
		64,			128,		256,		512,
		_1K,		_1K * 2,	_1K * 4,	_1K * 8,
		_1K * 16,	_1K * 32,	0,			0
	};

	u32	bor;
	u32	size;

	bor		= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bor_32);
	size	= bor & 0xF;
	size	= list[size];
	return(size);
}



/******************************************************************************
*
*	Function:	dev_write_startup
*
*	Purpose:
*
*		Perform any work or tests needed before initiating a write operation.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*	Returned:
*
*		0		All is well.
*		< 0		There was an error.
*
******************************************************************************/

int dev_write_startup(chan_data_t* chan)
{
	// There is nothing for us to do.
	return(0);
}



//*****************************************************************************
long pio_write_available(chan_data_t* chan, size_t count)
{
	u32		bor;
	long	bytes;
	u32		samples;
	u32		size;

	size	= _buffer_size(chan);
	bor		= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bor_32);

	if (bor & D15)		// FULL
		samples	= 0;
	else if (bor & D14)	// > 3/4 FULL
		samples	= 1;
	else if (bor & D12)	// EMPTY
		samples	= size;
	else if (bor & D13)	// < 1/4 FULL
		samples	= size * 3 / 4 + 1;
	else				// MEDIUM (1/4 to 3/4 full)
		samples	= size / 4;

	if (samples)
		bytes	= (samples - 1) * 4;	// Data loss is possible if it fills completely.
	else
		bytes	= 0;

	return(bytes);
}



//*****************************************************************************
long dma_write_available(chan_data_t* chan, size_t count)
{
	u32		bor;
	long	bytes;
	u32		samples;
	u32		size;

	size	= _buffer_size(chan);
	bor		= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bor_32);

	if (bor & D15)		// FULL
		samples	= 0;
	else if (bor & D14)	// > 3/4 FULL, which leaves too little free space
		samples	= 0;
	else if (bor & D12)	// EMPTY
		samples	= size;
	else if (bor & D13)	// < 1/4 FULL
		samples	= size * 3 / 4 + 1;
	else				// MEDIUM (1/4 to 3/4 full)
		samples	= size / 4;

	if (samples)
		bytes	= (samples - 1) * 4;	// Data loss is possible if it fills completely.
	else
		bytes	= 0;

	return(bytes);
}



/******************************************************************************
*
*	Function:	dma_write_work
*
*	Purpose:
*
*		Perform a write of the specified number of bytes.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*		mem		Put the data here.
*
*		count	What remains of the number of bytes that the application
*				requested.
*
*		st_end	The "system ticks" time at which we timeout. If this is zero,
*				then we ignore this.
*
*	Returned:
*
*		>= 0	The number of bytes written.
*		< 0		There was an error.
*
******************************************************************************/

long dma_write_work(
	chan_data_t*	chan,
	const os_mem_t*	mem,
	size_t			count,
	os_time_tick_t	st_end)
{
	u32		dpr;
	u32		mode;
	long	qty;
	long	samples	= count / 4;

	if (samples < chan->tx.pio_threshold)
	{
		qty	= gsc_write_pio_work_32_bit(chan, mem, count, st_end);
	}
	else
	{
		mode	= GSC_DMA_MODE_BLOCK_DMA
				| GSC_DMA_MODE_SIZE_32_BITS
				| GSC_DMA_MODE_INPUT_ENABLE
				| GSC_DMA_MODE_BURSTING_LOCAL
				| GSC_DMA_MODE_INTERRUPT_WHEN_DONE
				| GSC_DMA_MODE_LOCAL_ADRESS_CONSTANT
				| GSC_DMA_MODE_PCI_INTERRUPT_ENABLE;

		dpr		= GSC_DMA_DPR_HOST_TO_BOARD
				| GSC_DMA_DPR_END_OF_CHAIN
				| GSC_DMA_DPR_TERMINAL_COUNT_IRQ;

		qty		= gsc_dma_perform(	chan,
									&chan->tx,
									st_end,
									GSC_DMA_CAP_DMA_WRITE,
									mode,
									dpr,
									mem,
									count);
	}

	return(qty);
}



/******************************************************************************
*
*	Function:	dmdma_write_available
*
*	Purpose:
*
*		Check to see how many bytes are available for writing.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*		count	What remains of the number of bytes that the application
*				requested.
*
*	Returned:
*
*		>= 0	The number of bytes available for writing.
*		< 0		There was an error.
*
******************************************************************************/

long dmdma_write_available(chan_data_t* chan, size_t count)
{
	// The 16AO4MF does not support DMDMA.
	return(-EPERM);
}



/******************************************************************************
*
*	Function:	dmdma_write_work
*
*	Purpose:
*
*		Perform a write of the specified number of bytes.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*		mem		Put the data here.
*
*		count	What remains of the number of bytes that the application
*				requested.
*
*		st_end	The "system ticks" time at which we timeout. If this is zero,
*				then we ignore this.
*
*	Returned:
*
*		>= 0	The number of bytes written.
*		< 0		There was an error.
*
******************************************************************************/

long dmdma_write_work(
	chan_data_t*	chan,
	const os_mem_t*	mem,
	size_t			count,
	os_time_tick_t	st_end)
{
	// The 16AO4MF does not support DMDMA.
	return(-EPERM);
}


