// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/dout/dio.c $
// $Rev: 43281 $
// $Date: 2018-07-09 17:41:48 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_bin_show
*
*	Purpose:
*
*		Display the given data in binary.
*
*	Arguments:
*
*		data	The data to show.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _bin_show(u8 data)
{
	u8	bit;

	for (bit = 0x80; bit; bit /= 2)
	{
		if (bit & data)
			printf("1");
		else
			printf("0");
	}

	fflush(stdout);
}



/******************************************************************************
*
*	Function:	_dev_init
*
*	Purpose:
*
*		Initialize the board.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_init(int fd)
{
	int	errs;
	int	ret;

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_INITIALIZE, NULL);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_port_dir_set_out
*
*	Purpose:
*
*		Set GPIO 0 to an output.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_port_dir_set_out(int fd)
{
	int	errs;
	int	ret;
	u32	val		= AO4MF_DIO_DIR_OUT;

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_DIO_DIR, &val);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_port_out
*
*	Purpose:
*
*		Write to the Digital I/O Port output.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		data	The data to write.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_port_out(int fd, u8 data)
{
	int	errs;
	int	ret;
	u32	val		= data;

	_bin_show(data);
	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_DIO_TX, &val);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_output
*
*	Purpose:
*
*		Write to the Digital I/O outputs.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		data	The data to write.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_output(int fd, u8 data)
{
	int	errs	= 0;

	printf("    ");
	errs	+= _dev_dio_port_out(fd, data);
	printf("\n");
	gsc_time_sleep_ms(250);
	return(errs);
}



/******************************************************************************
*
*	Function:	dio_out
*
*	Purpose:
*
*		Perform a GPIO output test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int dio_out(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;
	s32	val;

	for (;;)	// A convenience loop
	{
		gsc_label("GPIO Output");

		val	= AO4MF_QUERY_REG_DIOPCR;
		ret	= ao4mf_ioctl(fd, AO4MF_IOCTL_QUERY, &val);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%d. ERROR: ao4mf_ioctl())\n", __LINE__);
			break;
		}

		if (val == 0)
		{
			printf("SKIPPED  (This channel doesn't support digital I/O.)\n");
			break;
		}

		errs	+= _dev_init(fd);
		errs	+= _dev_dio_port_dir_set_out(fd);

		if (errs)
		{
			printf("FAIL <---\n");
			break;
		}

		printf("\n");
		printf("    DIO Port\n");
		printf("    76543210\n");
		printf("    ========\n");

		errs	+= _dev_dio_output(fd, 0x00);
		errs	+= _dev_dio_output(fd, 0xFF);
		errs	+= _dev_dio_output(fd, 0x00);
		errs	+= _dev_dio_output(fd, 0xFF);
		errs	+= _dev_dio_output(fd, 0x00);

		for (i = 0; i <= 7; i++)
			errs	+= _dev_dio_output(fd, 0x001 << i);

		errs	+= _dev_dio_output(fd, 0x00);

		for (i = 7; i >= 0; i--)
			errs	+= _dev_dio_output(fd, 0x001 << i);

		errs	+= _dev_dio_output(fd, 0x00);
		errs	+= _dev_dio_output(fd, 0xFF);
		errs	+= _dev_dio_output(fd, 0x00);
		errs	+= _dev_dio_output(fd, 0xFF);
		errs	+= _dev_dio_output(fd, 0x00);

		gsc_label_level_inc();
		errs	+= _dev_init(fd);
		gsc_label("Status");
		printf("%s\n", errs ? "FAIL <---" : "PASS");
		gsc_label_level_dec();
		break;
	}

	return(errs);
}



