// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/id/id.c $
// $Rev: 43283 $
// $Date: 2018-07-09 17:42:18 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static int	_query_list[100];



//*****************************************************************************
static int _show_eol(int errs)
{
	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
static int _show_u32_hex(int* fd, const char* name, u32* val, const char* format)
{
	char	buf[32];
	int		errs	= 0;
	int		i;

	gsc_label(name);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, format, (long) val[i]);
		local_label_str(buf);
	}

	return(errs);
}



//*****************************************************************************
static int _show_reg(int* fd, u32* val, u32 reg, const char* name, const char* format)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
			errs	+= ao4mf_reg_read(fd[i], -1, 0, reg, &val[i]);
	}

	errs	+= _show_u32_hex(fd, name, val, format);
	return(errs);
}



//*****************************************************************************
static int _validate_same_s32(int* fd, s32* val)
{
	int	any		= 0;
	int	errs	= 0;
	s32	good	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			continue;
		}
		else if (any == 0)
		{
			any		= 1;
			good	= val[i];
		}
		else if (val[i] != good)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _validate_same_u32(int* fd, u32* val)
{
	int	any		= 0;
	int	errs	= 0;
	u32	good	= 0;
	int	i;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
		}
		else if (any == 0)
		{
			any		= 1;
			good	= val[i];
		}
		else if (val[i] != good)
		{
			errs++;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _query_list_set(int option)
{
	int	errs	= 0;

	if (option >= AO4MF_IOCTL_QUERY_LAST)
	{
		errs++;
	}
	else
	{
		errs	= 0;
		_query_list[option]++;
	}

	return(errs);
}



//*****************************************************************************
static int _query_read_s32(int* fd, s32* val, s32 query)
{
	int	errs	= 0;
	int	i;
	int	ret;
	s32	v;

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			v		= query;
			ret		= ao4mf_ioctl(fd[i], AO4MF_IOCTL_QUERY, &v);
			val[i]	= v;
			errs	= ret ? 1 : 0;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _query_bool_str(int* fd, int query, const char* title, int same, const char* yes, const char* no)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case 0:	local_label_str(no);	break;
			case 1:	local_label_str(yes);	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_bool(int* fd, int query, const char* title, int same)
{
	int	errs	= 0;

	errs	= _query_bool_str(fd, query, title, same, "Yes", "No");
	return(errs);
}



//*****************************************************************************
static int _query_count(int* fd)
{
	char	buf[32];
	char	err[64]	= "";
	int		errs	= 0;
	int		i;
	int		j;
	int		ret;
	s32		v;
	s32		val[4];

	gsc_label("Query Count");
	errs	+= _query_list_set(AO4MF_QUERY_COUNT);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _query_list_set", __LINE__);

	errs	+= _query_read_s32(fd, val, AO4MF_QUERY_COUNT);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _query_read_s32", __LINE__);

	errs	+= _validate_same_s32(fd, val);

	if ((errs) && (err[0] == 0))
		sprintf(err, "%d. _validate_same_s32", __LINE__);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, "%ld", (long) val[i]);
		local_label_str(buf);

		if (val[i] != AO4MF_IOCTL_QUERY_LAST)
		{
			errs++;

			if (err[0] == 0)
			{
				sprintf(err,
						"%d. incorrect count, ch[%d], expected %ld",
						__LINE__,
						i,
						(long) AO4MF_IOCTL_QUERY_LAST);
			}
		}

		// Verify that each query passes.

		for (j = 0; j < val[i]; j++)
		{
			v	= j;
			ret	= ao4mf_ioctl(fd[i], AO4MF_IOCTL_QUERY, &v);

			if (ret)
			{
				errs++;

				if (err[0] == 0)
				{
					sprintf(err,
							"%d. AO4MF_IOCTL_QUERY request[%d]",
							__LINE__,
							j);
				}
			}

			if (v == -1)
			{
				errs++;

				if (err[0] == 0)
				{
					sprintf(err,
							"%d. AO4MF_IOCTL_QUERY request[%d] results",
							__LINE__,
							j);
				}
			}
		}
	}

	if (errs)
		printf("FAIL <---");

	if ((errs) && (err[0]))
		printf("  (%s)", err);

	printf("\n");
	return(errs);
}



//*****************************************************************************
static int _query_device_type(int* fd)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label("Device Type");
	errs	+= _query_list_set(AO4MF_QUERY_DEVICE_TYPE);
	errs	+= _query_read_s32(fd, val, AO4MF_QUERY_DEVICE_TYPE);
	errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		switch (val[i])
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) val[i]);
				local_label_str(buf);
				break;

			case GSC_DEV_TYPE_16AO4MF:	local_label_str("16AO4MF/S");	break;
		}
	}

	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _query_decimal(int* fd, int query, const char* title, const char* suffix, int same)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	s32		val[4];

	gsc_label(title);
	errs	+= _query_list_set(query);
	errs	+= _query_read_s32(fd, val, query);

	if (same)
		errs	+= _validate_same_s32(fd, val);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] == -1)
		{
			local_label_str("----");
			continue;
		}

		sprintf(buf, "%ld", (long) val[i]);
		local_label_str(buf);
	}

	if (errs)
		_show_eol(errs);
	else
		printf("%s\n", suffix);

	return(errs);
}



//*****************************************************************************
static int _query_list_validate(void)
{
	int	err;
	int	errs	= 0;
	int	i;

	// Missed entries.

	for (err = 0, i = 0; i < AO4MF_IOCTL_QUERY_LAST; i++)
	{
		if (_query_list[i] == 0)
		{
			if (err == 0)
			{
				err++;
				gsc_label("Query Options Missed");
				printf("FAIL <---  (%d", i);
			}
			else
			{
				err++;
				printf(", %d", i);
			}
		}
	}

	if (err)
	{
		errs++;
		printf(")\n");
	}

	// Duplicate entries.

	for (err = 0, i = 0; i <= AO4MF_IOCTL_QUERY_LAST; i++)
	{
		if (_query_list[i] > 1)
		{
			if (err == 0)
			{
				err++;
				gsc_label("Query Options Duplicated");
				printf("FAIL <---  (%d", i);
			}
			else
			{
				err++;
				printf(", %d", i);
			}
		}
	}

	if (err)
	{
		errs++;
		printf(")\n");
	}

	return(errs);
}



//*****************************************************************************
static int _show_didr(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_DIDR, "Device ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x9080)
				errs++;
		}
	}

	if (errs == 0)
		printf("(PCI9080)\n");
	else
		_show_eol(errs);

	return(errs);
}



//*****************************************************************************
static int _show_frr(int* fd)
{
	int	errs	= 0;
	u32	val[4];

	errs	+= _show_reg(fd, val, AO4MF_GSC_FRR, "Firmware Rev Reg", "0x%08lX");
	errs	+= _validate_same_u32(fd, val);
	_show_eol(errs);
	return(errs);
}



//*****************************************************************************
static int _show_sid(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_SID, "Subsystem ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x3064)
				errs++;
		}
	}

	if (errs == 0)
		printf("(16AO4MF/S)\n");
	else
		_show_eol(errs);

	return(errs);
}



//*****************************************************************************
static int _show_svid(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_SVID, "Sub Vendor ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x10B5)
				errs++;
		}
	}

	if (errs == 0)
		printf("(PLX)\n");
	else
		_show_eol(errs);

	return(errs);
}



//*****************************************************************************
static int _show_vidr(int* fd)
{
	int	errs	= 0;
	int	i;
	u32	val[4];

	errs	+= _show_reg(fd, val, GSC_PCI_9080_VIDR, "Vendor ID", "0x%04lX");
	errs	+= _validate_same_u32(fd, val);

	if (errs == 0)
	{
		for (i = 0; i <= 3; i++)
		{
			if (fd[i] == -1)
				continue;

			if (val[i] != 0x10B5)
				errs++;
		}
	}

	if (errs == 0)
		printf("(PLX)\n");
	else
		_show_eol(errs);

	return(errs);
}



//*****************************************************************************
static int _register_map_list_gsc(int* fd)
{
	char					buf[32];
	int						ch;
	int						err;
	int						errs	= 0;
	int						i;
	const gsc_reg_def_t*	def;
	int						fd_tmp[4];
	s32						supp[4];
	u32						val[4]	= { 0, 0, 0, 0 };

	printf("\n");

	for (i = 0;; i++)
	{
		err	= 0;
		def	= ao4mf_reg_get_def_index(i);

		if (def == NULL)
			break;

		gsc_label(def->name);

		// Read the register values.

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] != -1)
				err	+= ao4mf_reg_read(fd[ch], -1, 0, def->reg, &val[ch]);
		}

		// Report the register values.
		fd_tmp[0]	= fd[0];
		fd_tmp[1]	= fd[1];
		fd_tmp[2]	= fd[2];
		fd_tmp[3]	= fd[3];

		for (ch = 0; ch <= 3; ch++)
		{
			if (fd[ch] == -1)
			{
				local_label_str("----");
				continue;
			}

			supp[ch]	= 1;

			if (def->ask_support)
			{
				supp[ch]	= (def->decode)(fd[ch], 1, 0, 0);

				if (supp[ch] == 0)
				{
					fd_tmp[ch]	= -1;	// don't compare value if unsupported
					local_label_str("NA");
				}
			}

			if (supp[ch])
			{
				sprintf(buf, "0x%08lX", (long) val[ch]);
				local_label_str(buf);
			}
		}

		// Perform channel value comparisons.

		if (def->ask_support)
			err	+= _validate_same_s32(fd_tmp, supp);

		// These are not always the same.
		// err	+= _validate_same_u32(fd_tmp, val);

		if (err)
			printf("FAIL <---\n");
		else
			printf("%s\n", def->desc);

		errs	+= err;
	}

	return(errs);
}



//*****************************************************************************
static int _frr_detail(int fd)
{
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	int	errs	= 0;

	printf("\n");
	ptr	= ao4mf_reg_get_def_id(AO4MF_GSC_FRR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(fd, list, 1, ao4mf_reg_read);
	}
	else
	{
		errs++;
		printf(	"FAIL <---  (%d. ao4mf_reg_get_def_id, AO4MF_GSC_FRR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
int id_device(int* fd)
{
	int	errs	= 0;
	int	i;

	// Identification =====================================
	gsc_label("Identification");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _show_vidr(fd);
	errs	+= _show_didr(fd);
	errs	+= _show_svid(fd);
	errs	+= _show_sid(fd);
	errs	+= _show_frr(fd);

	gsc_label_level_dec();

	// Features ==============================================
	gsc_label("Device Features");
	printf("\n");
	memset(_query_list, 0, sizeof(_query_list));

	gsc_label_level_inc();

	errs	+= _query_device_type(fd);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_AUT_CAL_MS,	"Auto-Cal",			"ms",	1	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_INIT_MS,		"Initialize",		"ms",	1	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_CHANNEL_QTY,	"Channels",			"",		1	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_INDEX_CHANNEL,	"Channel Index",	"",		0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_INDEX_DEVICE,	"Device Index",		"",		0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_FSAMP_MIN,		"Fsamp Minimum",	"S/S",	0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_FSAMP_MAX,		"Fsamp Maximum",	"S/S",	0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_FREF_MASTER,	"Master Reference",	"Hz",	0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_NRATE_MIN,		"Nrate Minimum",	"",		0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_NRATE_MAX,		"Nrate Maximum",	"",		0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_FREF_ADJ,		"Adjustable Ref.",	"Hz",	0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_NCLK_MIN,		"Nclk Minimum",		"",		0	);
	errs	+= _query_decimal	(fd, AO4MF_QUERY_NCLK_MAX,		"Nclk Maximum",		"",		0	);
	errs	+= _query_bool		(fd, AO4MF_QUERY_REG_DIOPCR,	"Reg: DIOPCR",				0	);
	errs	+= _query_bool		(fd, AO4MF_QUERY_SYNC_CLK_OUT,	"SYNC/CLK OUT",				0	);
	errs	+= _query_count		(fd);

	gsc_label_level_dec();

	errs	+= _query_list_validate();
	errs	+= _register_map_list_gsc(fd);

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] != -1)
		{
			errs	+= _frr_detail(fd[i]);
			break;
		}
	}

	printf("\n");
	return(errs);
}


