// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/id/main.c $
// $Rev: 43283 $
// $Date: 2018-07-09 17:42:18 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define COLUMN_WIDTH	14



// variables ******************************************************************

static	int	_def_index		= 0;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[64];
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	int		k;

	printf("id - Identify the specified board\n");
	printf("USAGE: id <index>\n");
	printf("  index  The zero based index of the device to access.\n");

	gsc_label("Arguments");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "s" : "");
	gsc_label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		gsc_label(buf);
		printf("%s\n", argv[i]);
	}

	gsc_label_level_dec();

	for (i = 1; i < argc; i++)
	{
		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_index	= k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: #%d = '%s'\n", i, argv[i]);
		break;
	}

	return(errs);
}



//*****************************************************************************
static void _column_headings(int index)
{
	char	buf[COLUMN_WIDTH + 1];
	int		chan;
	int		i;

	// Print the column titles.
	gsc_label("Operation");

	for (i = 0; i <= 3; i++)
	{
		chan	= (index * 4) + i;
		sprintf(buf, "Channel %i", chan);
		local_label_str(buf);
	}

	printf("\n");

	// Print the column spacers.
	gsc_label("=========");

	for (i = 0; i <= 3; i++)
		local_label_str("=========================");

	printf("\n");
}



//*****************************************************************************
static int _channels_open(int index, int* fd)
{
	int	chan;
	int	errs	= 0;
	int	i;
	int	ret;

	gsc_label("Opening Channels");

	for (i = 0; i <= 3; i++)
	{
		chan	= (index * 4) + i;
		ret		= ao4mf_open(chan, 0, &fd[i]);

		if (ret)
		{
			errs++;
			local_label_str("FAIL <---");
		}
		else
		{
			local_label_str("PASS");
		}
	}

	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
static int _channels_close(int* fd)
{
	int	errs	= 0;
	int	i;
	int	ret;

	gsc_label("Closing Channels");

	for (i = 0; i <= 3; i++)
	{
		if (fd[i] < 0)
		{
			local_label_str("----");
			continue;
		}

		ret	= ao4mf_close(fd[i]);

		if (ret < 0)
		{
			errs++;
			local_label_str("FAIL <---");
		}
		else
		{
			local_label_str("PASS");
		}
	}

	if (errs)
		printf("FAIL <---");

	printf("\n");
	return(errs);
}



//*****************************************************************************
static void _show_access_index(int index)
{
	gsc_label("Accessing Device");
	printf("%d\n", index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall operation of the application.
*
*	Arguments:
*
*		argc	The number of command line arguments.
*
*		argv	The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	int		fd[4];
	long	hours;
	int		index;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty			= 0;
	int		ret			= EXIT_SUCCESS;
	long	secs;
	time_t	test;
	long	tests		= 0;

	for (;;)
	{
		gsc_label_init(26);
		test	= time(NULL);
		errs	= _parse_args(argc, argv);

		if (errs)
			break;

		// Introduction

		os_id_host();
		_show_time_stamp();
		_show_access_index(_def_index);
		errs	+= ao4mf_init_util(1);
		errs	+= os_id_driver(ao4mf_open, ao4mf_read, ao4mf_close);
		errs	+= ao4mf_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to device (here, only the first)

		index	= _def_index * 4;
		errs	+= ao4mf_open_util(index, 0, -1, 1, &fd[0]);

		if (errs == 0)
		{
			// Identify the device, then close access
			errs	+= ao4mf_id_device(fd[0], -1, 1);
			errs	+= ao4mf_close_util(fd[0], -1, 1);
		}

		printf("\n");

		// Open access to devices (here, all four)

		_column_headings(_def_index);
		errs	+= _channels_open(_def_index, fd);

		if ((fd[0] >= 0) ||
			(fd[1] >= 0) ||
			(fd[2] >= 0) ||
			(fd[3] >= 0))
		{
			// Device identification
			// Perform testing

			errs	+= id_device(fd);

			// Close device access

			errs	+= _channels_close(fd);
		}

		// End of processing.

		now		= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);
		break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}



//*****************************************************************************
void local_label_str(const char* str)
{
	char	buf[COLUMN_WIDTH + 1];
	int		len;

	memset(buf, ' ', sizeof(buf));

	if (str == NULL)
		str	= "";

	len	= strlen(str);

	if (len > COLUMN_WIDTH)
		len	= COLUMN_WIDTH;

	strncpy(buf, str, len);
	buf[COLUMN_WIDTH]		= 0;
	buf[COLUMN_WIDTH - 1]	= ' ';
	buf[COLUMN_WIDTH - 2]	= ' ';
	printf("%s", buf);
}


