// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/irq/irq.c $
// $Rev: 43284 $
// $Date: 2018-07-09 17:42:34 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _initialize(int fd)
{
	int	errs	= 0;

	errs	+= ao4mf_irq_sel	(fd, -1, 0, AO4MF_IRQ_SEL_NONE_INIT_DONE, NULL);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_initialize_test(int fd)
{
	int	errs	= 0;

	gsc_label("Initialize Done");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_INIT_DONE, _initialize);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _auto_cal(int fd)
{
	int	errs	= 0;

	errs	+= ao4mf_irq_sel	(fd, -1, 0, AO4MF_IRQ_SEL_AUTO_CAL_DONE, NULL);
	errs	+= ao4mf_auto_cal	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_auto_cal_done_test(int fd)
{
	int	errs	= 0;

	gsc_label("Auto-Cal Done");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_AUTO_CAL_DONE, _auto_cal);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _out_buf_empty(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 100; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x8000);

	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_AO_BUF_EMPTY,	NULL);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, AO4MF_AO_CLK_ENABLE_YES, NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_out_buf_empty_test(int fd)
{
	int	errs	= 0;

	gsc_label("Output Buffer Empty");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_AO_BUF_EMPTY, _out_buf_empty);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _out_buf_low_qtr(int fd)
{
	int	errs	= 0;
	int	i;

	errs	+= ao4mf_ao_buf_size	(fd, -1, 0, AO4MF_AO_BUF_SIZE_128, NULL);

	for (i = 0; i < 100; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0);

	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_AO_BUF_1Q_FULL, NULL);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, AO4MF_AO_CLK_ENABLE_YES, NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_out_buf_low_qtr_test(int fd)
{
	int	errs	= 0;

	gsc_label("Output Buffer Low Quarter");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_AO_BUF_1Q_FULL, _out_buf_low_qtr);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _out_buf_high_qtr(int fd)
{
	int	errs	= 0;
	int	i;

	errs	+= ao4mf_ao_buf_size	(fd, -1, 0, AO4MF_AO_BUF_SIZE_128, NULL);
	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_AO_BUF_3Q_FULL, NULL);

	for (i = 0; i < 100; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0);

	return(errs);
}



//*****************************************************************************
static int _irq_out_buf_high_qtr_test(int fd)
{
	int	errs	= 0;

	gsc_label("Output Buffer High Quarter");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_AO_BUF_3Q_FULL, _out_buf_high_qtr);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _burst_trigger_ready(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 100; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0);

	errs	+= ao4mf_burst_mode		(fd, -1, 0, AO4MF_BURST_MODE_ENABLE, NULL);
	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_BURST_TRIG_READY,	NULL);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, AO4MF_AO_CLK_ENABLE_YES, NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_burst_trigger_ready_test(int fd)
{
	int	errs	= 0;

	gsc_label("Burst Trigger Ready");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_BURST_TRIG_READY, _burst_trigger_ready);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 100; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x8000);

	errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x18000);

	errs	+= ao4mf_ao_buf_mode	(fd, -1, 0, AO4MF_AO_BUF_MODE_CIRC, NULL);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, AO4MF_AO_CLK_ENABLE_YES, NULL);
	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_LOAD_READY, NULL);
	errs	+= ao4mf_ao_buf_load_req(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_LOAD_READY, _load_ready);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready_end(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1000; i++)
		errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x8000);

	errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x18000);

	errs	+= ao4mf_ao_buf_mode	(fd, -1, 0, AO4MF_AO_BUF_MODE_CIRC, NULL);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, AO4MF_AO_CLK_ENABLE_YES, NULL);
	errs	+= ao4mf_irq_sel		(fd, -1, 0, AO4MF_IRQ_SEL_LOAD_READY_END, NULL);
	errs	+= ao4mf_ao_buf_load_req(fd, -1, 0);
	errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x8000);
	errs	+= ao4mf_reg_write(fd, -1, 0, AO4MF_GSC_ODBR, 0x18000);
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_end_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready End");
	errs	+= ao4mf_initialize	(fd, -1, 0);
	errs	+= irq_wait_test	(fd, AO4MF_WAIT_GSC_LOAD_READY_END, _load_ready_end);
	errs	+= ao4mf_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
int irq_test(int fd)
{
	int	errs	= 0;

	gsc_label("IRQ");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq_initialize_test(fd);
	errs	+= _irq_auto_cal_done_test(fd);
	errs	+= _irq_out_buf_empty_test(fd);
	errs	+= _irq_out_buf_low_qtr_test(fd);
	errs	+= _irq_out_buf_high_qtr_test(fd);
	errs	+= _irq_burst_trigger_ready_test(fd);
	errs	+= _irq_load_ready_test(fd);
	errs	+= _irq_load_ready_end_test(fd);

	gsc_label_level_dec();
	return(errs);
}


