// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/txrate/tx.c $
// $Rev: 43288 $
// $Date: 2018-07-09 17:43:37 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define _1M	(1024L * 1024L)



// variables *******************************************************************

static u32	_tx[_1M];



/******************************************************************************
*
*	Function:	_rate_test
*
*	Purpose:
*
*		Perform for the write rate test.
*
*	Arguments:
*
*		fd		The handle to the board.
*
*		seconds	The minimum number of seconds to operate.
*
*		mb		The niminum number of mega-bytes to write.
*
*		send	The number of bytes to request with each write call.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _rate_test(int fd, int seconds, long mb, long send)
{
	struct timeval	begin;
	int				errs		= 0;
	struct timeval	minimum;
	struct timeval	now			= { 0, 0 };
	long			sent;
	long			sent_acc	= 0;
	long			sent_mb		= 0;
	double			total;
	double			us;

	gsc_label("Writing");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + seconds;
	minimum.tv_usec	= begin.tv_usec;

	memset(_tx, 0x01, sizeof(_tx));

	for (;;)
	{
		sent	= ao4mf_write(fd, _tx, send);

		if (sent < 0)
		{
			errs++;
			printf("FAIL <--- (ao4mf_write)\n");
			break;
		}

		gettimeofday(&now, NULL);
		sent_acc	+= sent;

		if (sent_acc >= _1M)
		{
			for (; sent_acc >= _1M;)
			{
				sent_acc	-= _1M;
				sent_mb++;
			}

			if (((now.tv_sec > minimum.tv_sec) ||
				((now.tv_sec == minimum.tv_sec) && (now.tv_usec >= minimum.tv_usec))) &&
				(sent_mb >= mb))
			{
				break;
			}
		}
	}

	if (errs == 0)
	{
		total	= (double) sent_mb * _1M + sent_acc;
		us		= (((double) now.tv_sec - begin.tv_sec) * 1000000)
				+ now.tv_usec - begin.tv_usec;
		printf(	"PASS  (");
		gsc_label_long_comma((long) total / 4);
		printf(" Samples, %.3f Seconds, ", us / 1000000);
		gsc_label_long_comma((long) (total / us * 1000000) / 4);
		printf(" S/S)\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	tx_rate_test
*
*	Purpose:
*
*		Perform an Tx Rate test - measure the write throughput.
*
*	Arguments:
*
*		fd		The handle to the board.
*
*		mb		The niminum number of mega-bytes to write.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_rate_test(int fd, long mb)
{
	int		errs	= 0;
	int		seconds	= 5;
	long	send	= sizeof(_tx);

	gsc_label ("Write Rate");
	printf("(%d second minimum, %ld MB minimum)\n", seconds, mb);
	gsc_label_level_inc();

	errs	+= _rate_test(fd, seconds, mb, send);

	gsc_label_level_dec();
	return(errs);
}



