// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/utils/intime/os_util_id.c $
// $Rev: 43207 $
// $Date: 2018-07-08 17:04:20 -0500 (Sun, 08 Jul 2018) $

// INtime: Utility: source file

#include "gsc_utils.h"



// #defines *******************************************************************

#define ISEOL(_c)	((((_c) == 0)		|| \
					  ((_c) == '\r')	|| \
					  ((_c) == '\n')) ? 1 : 0)



//*****************************************************************************
static void _id_host_os(void)
{
	SYSINFO	si;

	gsc_label("Operating System");
	printf("INtine for Windows\n");

	gsc_label("Kernel");
	CopyRtSystemInfo(&si);
	printf(	"%ld.%ld, Build %ld\n",
			(long) si.MajorVersion,
			(long) si.MinorVersion,
			(long) si.BuildNumber);
}



//*****************************************************************************
static void _id_host_processor(void)
{
	const char*	psz;
	SYSINFO		si;

	CopyRtSystemInfo(&si);

	gsc_label("Processor");
	printf(	"CPU Type %ld, CPU Model %ld\n",
			(long) si.CPUType,
			(long) si.CPUModel);

	switch (si.NodeType)
	{
		default:						psz	= "INTERNAL ERROR";	break;
		case NODE_TYPE_SINGLE_CPU:		psz	= "Single CPU";		break;
		case NODE_TYPE_SHARED_MP:		psz	= "Shared MP";		break;
		case NODE_TYPE_DEDICATED_MP:	psz	= "Dedicated MP";	break;
		case NODE_TYPE_REMOTE_MP:		psz	= "Remote MP";		break;
	}

	gsc_label("Node Type");
	printf("%s\n", psz);
}



/******************************************************************************
*
*	Function:	os_id_host
*
*	Purpose:
*
*		Identify the host os and machine.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void os_id_host(void)
{
	gsc_label("Host");
	printf("\n");
	gsc_label_level_inc();

	_id_host_os();
	_id_host_processor();

	gsc_label_level_dec();
}



//*****************************************************************************
int os_id_driver(
	int	(*dev_open)(int index, int share, int* fd),
	int	(*dev_read)(int fd, void* dst, size_t size),
	int	(*dev_close)(int fd))
{
	char	buf[1024];
	int		errs	= 0;
	int		fd;
	int		got;
	char*	ptr;
	int		ret;

	for (;;)	// A convenience loop.
	{
		gsc_label("Driver");

		if ((dev_open == NULL) ||
			(dev_read == NULL) ||
			(dev_close == NULL))
		{
			errs++;
			printf("FAIL <---  (Invalid argument(s).)\n");
			break;
		}

		ret	= (dev_open)(-1, 0, &fd);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (Driver mailbox access error.)\n");
			break;
		}

		memset(buf, 0, sizeof(buf));
		got	= dev_read(fd, buf, sizeof(buf));
		dev_close(fd);

		if (got <= 0)
		{
			errs++;
			printf("FAIL <---  (Driver mailbox read error.)\n");
			break;
		}

		buf[sizeof(buf) - 1]	= 0;

		// Version
		printf("Version ");
		ptr	= strstr(buf, "version");

		if (ptr == NULL)
		{
			errs++;
			printf("FAIL <---  (Driver mailbox content error.)\n");
			break;
		}

		for (ptr += 9; !ISEOL(ptr[0]); ptr++)
			printf("%c", ptr[0]);

		printf("\n");
		break;
	}

	return(errs);
}


