// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/utils/intime/os_util_sem.c $
// $Rev: 43207 $
// $Date: 2018-07-08 17:04:20 -0500 (Sun, 08 Jul 2018) $

// INtime: Utility: source file

#include "gsc_utils.h"



//*****************************************************************************
int os_sem_create(os_sem_t* sem)
{
	int	ret;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else
	{
		// Create a new, one unit semaphore.
		ret	= os_sem_create_qty(sem, 1, 1);
	}

	return(ret);
}



//*****************************************************************************
int os_sem_create_qty(os_sem_t* sem, int cap, int put)
{
	int	ret;

	if ((sem == NULL) || (cap < 1) || (put < 0) || (put > cap) || (put > 65535))
	{
		// A parameter is invalid.
		ret	= -EINVAL;
	}
	else
	{
		memset(sem, 0, sizeof(os_sem_t));
		sem->handle	= CreateRtSemaphore(put, cap, PRIORITY_QUEUING);

		if (sem->handle == BAD_RTHANDLE)
		{
			memset(sem, 0, sizeof(os_sem_t));
			ret	= - (int) GetLastRtError();
		}
		else
		{
			ret	= 0;
		}
	}

	return(ret);
}



//*****************************************************************************
int os_sem_destroy(os_sem_t* sem)
{
	int		ret;
	BOOLEAN	tst;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if ((sem->handle) && (sem->handle != BAD_RTHANDLE))
	{
		tst	= DeleteRtSemaphore(sem->handle);

		if (tst == FALSE)
		{
			ret	= - (int) GetLastRtError();
		}
		else
		{
			memset(sem, 0, sizeof(os_sem_t));
			ret	= 0;
		}
	}
	else
	{
		memset(sem, 0, sizeof(os_sem_t));
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
int os_sem_lock(os_sem_t* sem)
{
	DWORD	dw;
	int		ret;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if ((sem->handle) && (sem->handle != BAD_RTHANDLE))
	{
		// Gain access to the semaphore.
		dw	= WaitForRtSemaphore(sem->handle, 1, WAIT_FOREVER);

		if (dw == WAIT_FAILED)
			ret	= - (int) GetLastRtError();
		else
			ret	= 0;
	}
	else
	{
		// The semaphore was never created.
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
int os_sem_unlock(os_sem_t* sem)
{
	int		ret;
	BOOLEAN	tst;

	if (sem == NULL)
	{
		// The pointer is NULL.
		ret	= -EINVAL;
	}
	else if ((sem->handle) && (sem->handle != BAD_RTHANDLE))
	{
		// Release access to the semaphore.
		tst	= ReleaseRtSemaphore(sem->handle, 1);

		if (tst == FALSE)
			ret	= - (int) GetLastRtError();
		else
			ret	= 0;
	}
	else
	{
		// The semaphore was never created.
		ret	= -EINVAL;
	}

	return(ret);
}


